/*
 * database.h
 *
 * Copyright (C) 2009 Tim Marston <edam@waxworlds.org>
 *
 * This file is part of sqlite3cc (hereafter referred to as "this program").
 * See http://www.waxworlds.org/edam/software/sqlite3cc for more information.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SQLITE3CC_DATABASE_H_
#define SQLITE3CC_DATABASE_H_


#include <sqlite3.h>
#include <boost/utility.hpp>


namespace sqlite
{


class basic_statement;
class transaction;


class database
	:
	private boost::noncopyable
{
//______________________________________________________________________________
//                                                                 instantiation
public:

	/**
	 * Constructor that specifies the filename of a database to open.
	 * @param filename the filename of the database file to open
	 * @throws database_error on error
	 */
	explicit database(
		const std::string &filename );

	explicit database();
	virtual ~database();

//______________________________________________________________________________
//                                                              public interface
public:

	/**
	 * Open a database.
	 * @param filename the filename of the database file to open
	 * @param flags flags appropriate for sqlite3_open_v2()
	 * @returns an sqlite error code
	 * @see sqlite3_open_v2()
	 */
	int open(
		const std::string &filename,
		int flags = SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE );

	/**
	 * Close an open database.
	 * @throws database_error on error
	 * @see sqlite3_close()
	 */
	void close();

	/**
	 * Execute an SQL statement. An exception is thrown on error.
	 * @param sql an SQL statement in UTF-8
	 * @see sqlite3_exc()
	 */
	void exec(
		const std::string &sql );

	/**
	 * Sets a default busy handler which will
	 * wait for the specified number of milliseconds.
	 * @param milliseconds number of milliseconds to wait
	 * @returns an sqlite error code
	 * @see sqlite3_busy_timeout()
	 */
	int busy_timeout(
		int duration );

//______________________________________________________________________________
//                                                                implementation
private:

	friend class basic_statement;
	friend class command;

	/** the database handle */
	sqlite3 *_handle;

};


} // namespace sqlite


#endif /* SQLITE3CC_DATABASE_H_ */
